@@ -107,7 +107,7 @@ static char const *help_unichar = {
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "UNICHAR(65) equals A.\n"
-	   "UNICHAR(960) equals a small greek pi.\n"
+	   "UNICHAR(960) equals a small Greek pi.\n"
 	   "\n"
 	   "@SEEALSO=CHAR,UNICODE,CODE")
 };
@@ -613,7 +613,7 @@ static char const *help_fixed = {
 	   "@EXAMPLES=\n"
 	   "FIXED(1234.567,2) equals \"1,234.57\".\n"
 	   "\n"
-	   "@SEEALSO=")
+	   "@SEEALSO=TEXT, VALUE, DOLLAR")
 };
 
 static GnmValue *
@@ -806,7 +806,7 @@ static char const *help_text = {
 	   "TEXT(3.223,\"$0.00\") equals \"$3.22\".\n"
 	   "TEXT(date(1999,4,15),\"mmmm, dd, yy\") equals \"April, 15, 99\".\n"
 	   "\n"
-	   "@SEEALSO=DOLLAR")
+	   "@SEEALSO=DOLLAR, FIXED, VALUE")
 };
 
 static GnmValue *
@@ -1131,7 +1131,7 @@ const GnmFuncDescriptor string_functions[] = {
         { "unicode",    "S",     N_("text"),                    &help_unicode,
 	  gnumeric_unicode, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_BASIC },
-        { "concatenate", 0,      N_("text,text,"),            &help_concatenate,
+        { "concatenate", NULL,   N_("text,text,"),            &help_concatenate,
 	  NULL, gnumeric_concatenate, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "dollar",     "f|f",   N_("num,decimals"),            &help_dollar,
