@@ -1,4 +1,4 @@
-#
+# TODO :
 # Change this file to read in other Perl files and not register the
 # example function!!!
 #
@@ -13,8 +13,8 @@ sub foobar
 }
 
 $help_foobar = <<'EOS';
-@FUNCTION=ADDER
-@SYNTAX=ADDER(a,b)
+@FUNCTION=PERL_ADDER
+@SYNTAX=PERL_ADDER(a,b)
 @DESCRIPTION=
 Adds two numbers. It is just an example function.
 EOS
