@@ -24,6 +24,8 @@
 #ifndef GNUMERIC_SOLVER_H
 #define GNUMERIC_SOLVER_H 1
 
+#ifdef ENABLE_SOLVER
+
 #include "gnumeric.h"
 #include "numbers.h"
 
@@ -56,7 +58,7 @@ typedef enum {
 
 typedef gpointer SolverProgram;
 
-
+
 /*
  * Solver's API for LP solving algorithms
  */
@@ -98,7 +100,7 @@ typedef void
 typedef int
         (solver_lp_get_iterations_fn)   (SolverProgram p);
 
-
+
 typedef struct {
         const char                    *name;
         solver_init_fn                *init_fn;
@@ -137,12 +139,14 @@ struct _SolverOptions {
         gboolean            performance_report;
         gboolean            program_report;
         gboolean            dual_program_report;
+	gboolean            add_scenario;
+	gchar               *scenario_name;
         SolverAlgorithmType algorithm;
 };
 
 struct _SolverConstraint {
-        CellPos              lhs;		/* left hand side */
-        CellPos              rhs;  		/* right hand side */
+        GnmCellPos           lhs;		/* left hand side */
+        GnmCellPos           rhs;  		/* right hand side */
         gint                 rows;              /* number of rows */
         gint                 cols;              /* number of columns */
         SolverConstraintType type;	        /* <=, =, >=, int, bool */
@@ -151,7 +155,7 @@ struct _SolverConstraint {
 
 struct _SolverParameters {
         SolverProblemType  problem_type;
-        Cell               *target_cell;
+        GnmCell            *target_cell;
         GSList		   *input_cells;
         GSList             *constraints;
         char               *input_entry_str;
@@ -196,7 +200,7 @@ typedef struct {
         gboolean         ilp_flag;   /* This is set if the problem has INT
 				      * constraints.  Some reports cannot
 				      * be created if there are any. */
-        Cell             **input_cells_array;
+        GnmCell          **input_cells_array;
         SolverConstraint **constraints_array;
         gnm_float       *obj_coeff;
         gnm_float       **constr_coeff;
@@ -204,7 +208,7 @@ typedef struct {
         SolverParameters *param;
 } SolverResults;
 
-
+
 /**************************************************************************
  *
  * The API functions to the Solver tool.
@@ -214,12 +218,12 @@ typedef struct {
  * the results of the run.  Note that it should be freed when no longer
  * needed (by calling solver_results_free). */
 SolverResults    *solver               (WorkbookControl *wbc, Sheet *sheet,
-					gchar **errmsg);
+					const gchar **errmsg);
 
 /* Creates the Solver's reports. */
 gchar *          solver_reports        (WorkbookControl *wbc, Sheet *sheet,
 					SolverResults *res,
-					gboolean answer, gboolean sensitivity, 
+					gboolean answer, gboolean sensitivity,
 					gboolean limits, gboolean performance,
 					gboolean program, gboolean dual);
 
@@ -245,10 +249,10 @@ void             solver_results_free   (SolverResults *res);
 
 /* Returns a pointer to the target cell of the model attached to the
  * given sheet. */
-Cell*            solver_get_target_cell (Sheet *sheet);
+GnmCell		*solver_get_target_cell (Sheet *sheet);
 
 /* Returns a pointer to a input variable cell. */
-Cell*            solver_get_input_var (SolverResults *res, int n);
+GnmCell		*solver_get_input_var (SolverResults *res, int n);
 
 /* Retruns a pointer to a constraint. */
 SolverConstraint* solver_get_constraint (SolverResults *res, int n);
@@ -256,6 +260,18 @@ SolverConstraint* solver_get_constraint (SolverResults *res, int n);
 void              solver_insert_cols    (Sheet *sheet, int col, int count);
 void              solver_insert_rows    (Sheet *sheet, int row, int count);
 void              solver_delete_rows    (Sheet *sheet, int row, int count);
-void              solver_delete_cols    (Sheet *sheet, int row, int count);
+void              solver_delete_cols    (Sheet *sheet, int col, int count);
+
+#else /* !ENABLE_SOLVER */
+
+#define solver_param_new() NULL
+#define solver_lp_copy(src_param, new_sheet) NULL
+#define solver_param_destroy(param)
+#define solver_insert_cols(sheet, col, count)
+#define solver_insert_rows(sheet, row, count)
+#define solver_delete_cols(sheet, col, count)
+#define solver_delete_rows(sheet, row, count)
+
+#endif
 
 #endif
